﻿using RobotInLabuteny;
using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace WindowsFormsApplication6
{
    public partial class Form1 : Form
    {
        private Pen polygonPen; //для рисования контура многоугольника
        private SolidBrush trianBrush; //для рисования треугольников

        private Graphics g;
        private Bitmap bitmap;

        private Polygon polygon; //многоугольник
        private int drawCount = 0; //количество нарисованных треугольников в данный момент
        private Color[] colors; //цвета треугольников

        private float translateX, translateY, scale;


        public Form1()
        {
            InitializeComponent();
        }

        private void drawPolygon() //рисует многоугольник
        {
            g.TranslateTransform(translateX, translateY); //центрируем
            g.ScaleTransform(scale, scale); //масштабируем

            //рисуем контур
            PointF[] points = polygon.getPoints();
            g.DrawLines(polygonPen, points);
            g.DrawLine(polygonPen, points[0], points[points.Length - 1]);

            GraphicsPath p = new GraphicsPath();
            Triangle[] trians = polygon.getTriangles();

            if (trians == null)
                return;

            //рисуем drawCount треугольников
            for (int i = 0; i < drawCount; i++)
            {
                Triangle t = trians[i];

                p.Reset();
                p.AddLine(t.getA(), t.getB());
                p.AddLine(t.getB(), t.getC());
                p.AddLine(t.getC(), t.getA());

                trianBrush.Color = colors[i];
                g.FillPath(trianBrush, p);
            }
        }

        private void pictureBox1_Click(object sender, EventArgs e) //по щелчку
        {
            if (colors != null) //если триангуляция прошла успешно
            {
                drawCount++; //увеличиваем количество отображаемых треугольников
                if (drawCount > polygon.getTriangles().Length)
                    drawCount = 0;
                draw();
            }
        }

        private void draw()
        {
            bitmap = new Bitmap(pictureBox1.Width, pictureBox1.Height);
            g = Graphics.FromImage(bitmap);
            g.SmoothingMode = SmoothingMode.AntiAlias;

            drawPolygon();

            pictureBox1.Image = bitmap;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            polygonPen = new Pen(Color.Black, 1);
            trianBrush = new SolidBrush(Color.Red);

            float[] vertex = new float[]
            {
                248.65465f, 99.79547f,
                59.30233f, 437.65973f,
                335.534f, 321.62216f,
                83.8835f, 767.274f,
                378.47433f, 619.3895f,
                287.60056f, 1020.07654f,
                634.11926f, 1001.0913f,
                658.086f, 417.54724f,
                249.65326f, 596.4075f,
                613.14844f, 233.69086f,
                287.60056f, 228.69476f,
                622.1359f, 58.827484f,
                59.40804f, 71.653946f,
                30.957005f, 313.62842f
            }; //х и y координаты многоугольник против часовой стрелки

            //вершины многоугольника против часовой стрелки
            polygon = new Polygon(vertex);
            if (polygon.getTriangles() != null) //если триангуляция была проведена успешно
            {
                Triangle[] trians = polygon.getTriangles(); 
                colors = new Color[trians.Length];

                Random rand = new Random();
                for (int i = 0; i < colors.Length; i++) //заполняем палитру случайными цветами
                    colors[i] = Color.FromArgb(rand.Next(25, 225), rand.Next(25, 225), rand.Next(25, 225));
            }

        //разместим многоугольник на весь экран
            //масштабирование
            PointF[] points = polygon.getPoints();

            float minX = int.MaxValue, minY = int.MaxValue;
            float maxX = int.MinValue, maxY = int.MinValue;
            foreach (PointF p in points)
            {
                minX = Math.Min(minX, p.X);
                minY = Math.Min(minY, p.Y);
                maxX = Math.Max(maxX, p.X);
                maxY = Math.Max(maxY, p.Y);
            }

            float width = maxX - minX;
            float height = maxY - minY;

            float scaleX = pictureBox1.Width / width;
            float scaleY = pictureBox1.Height / height;
            scale = Math.Min(scaleX, scaleY) * 0.95f; //коэффициент масштабирования

            //центрирование многоугольника
            translateX = (pictureBox1.Width - width * scale) / 2 - minX * scale;
            translateY = (pictureBox1.Height - height * scale) / 2 - minY * scale;

            draw();
        }
    }
}
